/*
 * camcapture.h
 * This file is part of camcapture
 *
 * Copyright (C) 2012-2013 - Florian Schmitt <florian.schmitt@qikgrp.com>
 *
 * camcapture is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * camcapture is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with <program name>. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef CAMCAPTURE_H
#define CAMCAPTURE_H




/* #####################################################
 * STRUCT DEFS
 * ##################################################### 
*/

/**
 * @struct color_rgb
 * This struct defines a color
 * @typedef color_rgb_t
 * An color_rgb struct
*/
typedef struct color_rgb {
	int r; /**< Red value (0-255) */
	int g; /**< Green value (0-255) */
	int b; /**< Blue value (0-255) */
} color_rgb_t;

/**
 * @struct image_data
 * This struct defines an image object
 * @typedef image_data_t
 * An image_data struct
*/
typedef struct image_data {
	void *img;  /**< Pointer to the raw image data */
	int width;  /**< Image width in pixel */
	int height; /**< Image height in pixel */
} image_data_t;

/**
 * @struct image_point
 * This struct defines a point in an image
 * @typedef image_point_t
 * An image_point struct
*/
typedef struct image_point {
	int x; /**< x-position inside an image */
	int y; /**< y-position inside an image */
} image_point_t;

/**
 * @struct image_point_list
 * This struct is used to create linked lists of image points. 
 * Important: use the given functions to handle point lists
 * See: camcapture_create_point_list()
 * See: camcapture_destroy_point_list()
 * @typedef image_point_list_t
 * An image_point_list struct
*/
typedef struct image_point_list {
	image_point_t *data; /**< Data pointer to an image_point_t object */
	struct image_point_list *next; /**< The next linked list member */
} image_point_list_t;

/**
 * @struct image_circle
 * This struct represents a circle
 * @typedef image_circle_t
 * An image_circle struct
*/
typedef struct image_circle {
	int x; /**< x-position of center */
	int y; /**< y-position of center */
	int rad; /**< The circles radius */
} image_circle_t;





/* #####################################################
 * FROM imgutil.c
 * ##################################################### 
*/
/* extracts YCbCr-values from RGB-colors */
void rgb_to_ycbcr(const color_rgb_t *, int *, int *, int *);
/* copies one image into the other - doesn't allocate anything */
extern void camcapture_copy_image(const image_data_t*, image_data_t*);
/* Allocates a compatible image to preset into dest */
void camcapture_create_compatible_image(const image_data_t *preset, image_data_t *dest);
/* frees (if there is any) memory of destination and reallocates it. Then it copies the data from source. */
void camcapture_clone_image(const image_data_t*, image_data_t*);
/* Get a pixel at a given position */
void camcapture_get_pixel(image_data_t*, int, int, color_rgb_t*);
/* Free an image's data. Do never free images got from camcapture_get_frame() ! */
void camcapture_free_image_data(image_data_t*);






/* #####################################################
 * FROM capture.c
 * ##################################################### 
*/

/* get the current frames data into a frameinfo */
extern void camcapture_get_frame(image_data_t*);
/* startup everything */
extern int camcapture_init(const char*, unsigned int, unsigned int);
/* prepares for reading */
extern int camcapture_lock(void);
/* unlock after reading */
extern int camcapture_unlock(void);
/* free & cleanup */
extern void camcapture_destroy(void);







/* #####################################################
 * FROM framesave.c
 * ##################################################### 
*/

/* save a single frame to a pnm-file */
extern int camcapture_save_pnm(const image_data_t*, const char*);
/* save a single frame to a pnm-file, append framenumber to filname */
extern int camcapture_save_pnm_inc(const image_data_t*, const char*);







/* #####################################################
 * FROM imgdraw.c
 * ##################################################### 
*/
/* draw a solid line into the image */
extern void camcapture_draw_line(image_data_t*, int, int, int, int, color_rgb_t*);
/* draw a cross marker into the image */
extern void camcapture_draw_cross(image_data_t*, int, int, color_rgb_t*);
/* draw a cross marker into the image by an point object */
extern void camcapture_draw_cross_obj(image_data_t*, image_point_t*, color_rgb_t*);
/* draw a (not filled) rectangle */
extern void camcapture_draw_rect(image_data_t*, int, int, int, int, color_rgb_t*);
/* draw a filled rectangle */
extern void camcapture_draw_fillrect(image_data_t*, int, int, int, int, color_rgb_t*);
/* draw a circle */
extern void camcapture_draw_circle(image_data_t*, int, int, int, color_rgb_t*);
/* draw a circle object */
extern void camcapture_draw_circle_obj(image_data_t*, image_circle_t*, color_rgb_t*);
/* draw a hole list of points as crosses */
extern void camcapture_draw_point_list_crossed(image_data_t*, image_point_list_t*, color_rgb_t*);
/* draw a hole list of points as path */
extern void camcapture_draw_point_list_path(image_data_t*, image_point_list_t*, color_rgb_t*);
/* draw a hole point list with rectangles around the points */
extern void camcapture_draw_point_list_rects(image_data_t*, image_point_list_t*, int, color_rgb_t*);
/* sets an images pixel to a given color */
extern void camcapture_set_pixel(image_data_t*, int, int, color_rgb_t*);





/* #####################################################
 * FROM imgfx.c
 * ##################################################### 
*/
 
/* Extracts a color - creates a filter */
extern void camcapture_filter_color(image_data_t*, color_rgb_t*, int);
/* filters an image by luminescense */
extern void camcapture_filter_lumi(image_data_t*, int);
/* gets luminescense of a given point: (Y0+Y1)/2 */
extern int camcapture_get_luma_single(void*);
/* filters luminescense of a given point: (Y0+Y1)/2 */
extern int camcapture_filter_luma_single(void*, int);
/* checks if the color at a specific point is a specific one */
extern int  camcapture_filter_color_single(void *, int, int, int);
/* Calculate difference between two pictures */
extern void camcapture_filter_difference(image_data_t *destination,  image_data_t *src1, 
                                         image_data_t *src2, int sens);






/* #####################################################
 * FROM imganalysis.c
 * ##################################################### 
*/
 
/* looks for edges on x and stores then into a given image_point array */
extern int camcapture_edge_detection_x(image_data_t*, int, int, int, image_point_list_t* list);
/* looks for edges on x and stores then into a given image_point array */
extern int camcapture_edge_detection_x_col(image_data_t*, color_rgb_t*, int, int, int, image_point_list_t* list);
/* looks for edges (by luminescense) on x and stores then into a given image_point array */
extern int camcapture_edge_detection_x_dlum(image_data_t*, color_rgb_t*, int, int, int, image_point_list_t*);
/* check if a rect exceeds an threshold */
extern int camcapture_rect_threshold(image_data_t*, int, int, image_point_list_t*);







/* #####################################################
 * FROM lists.c
 * ##################################################### 
*/

/* creates a list of points (image_point_t) */
extern image_point_list_t *camcapture_create_point_list(void);
/* destroys a list of points (image_point_t) */
extern void camcapture_destroy_point_list(image_point_list_t *);
/* add a point to the point list and return the added image_point_list_t node */
extern image_point_list_t *camcapture_add_point_list(image_point_list_t*, image_point_t*);
/* append point to the point list and return the added image_point_list_t node - WARNING: SLOWLY! */
extern image_point_list_t *camcapture_append_point_list(image_point_list_t*, image_point_t*);
/* gets the next point from a list */
extern image_point_list_t *camcapture_next_point_list(image_point_list_t*);
/* Calculates the points median point and returns it */
extern image_point_t camcapture_point_list_median(image_point_list_t*);
/* kills every point that exceeds the distance from a given center */
extern int camcapture_point_kill_overshoots(image_point_list_t*, image_point_t, int);
/* kills points that have no large white area in their background */
extern int camcapture_point_kill_nonobjs(image_point_list_t*, image_data_t*, int);
/* kills points that have no large solid color background next to them*/
extern int camcapture_point_kill_nonobjs_col(image_point_list_t*, image_data_t*, int, int, color_rgb_t*);
/* calculate the points median delta (if the detected edge points are in a circle, this is the radius) */
extern int camcapture_point_list_median_delta(image_point_list_t*, image_point_t);
/* calculate the points maximum delta (if the detected edge points are in a circle, this is the radius) */
extern int camcapture_point_list_max_delta(image_point_list_t*, image_point_t);
/* Count list members */
extern int camcapture_point_list_count(image_point_list_t*);

/* some functions to get the max/min points */
extern int camcapture_point_list_max_x(image_point_list_t*);
extern int camcapture_point_list_min_x(image_point_list_t*);
extern int camcapture_point_list_max_y(image_point_list_t*);
extern int camcapture_point_list_min_y(image_point_list_t*);






/* #####################################################
 * FROM listsort.c
 * ##################################################### 
*/
/* sort list by a special circle-fit */
extern void camcapture_point_list_sort_by_circle(image_point_list_t*, image_point_t*, int);
/* sort list by point distance to each other */
extern void camcapture_point_list_sort_by_nextto(image_point_list_t*);









/* #####################################################
 * FROM mathutil.c
 * ##################################################### 
*/
/* limit a given double to integer values 0-255 */
int clamp(double);
/* calculate absolute value */
int absolute(int);
/* calculate distance between two values */
int deltaval(int, int);
/* calculate distance between two points */
extern int camcapture_math_distance(image_point_t*, image_point_t*);
/* calculate the distance of a ball from the image */
extern float camcapture_math_ball_distance(int, int);








/* #####################################################
 * FROM drawtext.c
 * ##################################################### 
*/
/* draws a single character at (x,y) */
extern void camcapture_draw_character(image_data_t*, int, int, char, color_rgb_t*);
/* draws a string at (x,y) */
extern void camcapture_draw_string(image_data_t*, int, int, char*, color_rgb_t*);



#endif /* CAMCAPTURE_H */
